/*------------------------------------------------*/
/* 08textury/textura/main.c                       */
#define _CRT_SECURE_NO_WARNINGS
#define GLEW_STATIC
#include <GL/glew.h>
#include <stdio.h>
#include <GLFW/glfw3.h>
#include <stdbool.h>
#include <math.h>
#include "linmath-extended.h"
#include "vertex-data.h"
#include "shader.h"
#include "update-angles.h"
#include "events.h"
#include "stb-image.h"

void initVariables(Shader *program);
void initBuffers(Shader *program);
void uninitBuffers(void);
void ticking(Shader * program, GLFWwindow* window);
void draw(Shader *program, GLFWwindow* window, GLuint *VAO);
void animate(void);

GLuint VAO[1], BO[3], TEX[1];
struct {
	GLuint uPMatrixLoc;
	GLuint uMVMatrixLoc;
	GLuint uAmbientColorLoc;
	GLuint texture1;
} variables;
mat4x4 rotationMatrix;
float x, y, z; // light position

int main(void)
{
	GLFWwindow* window;
	/* Initialize the library */
	if (!glfwInit()) {
		return -1;
	}
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	/* Create a windowed mode window and its OpenGL context */
	window = glfwCreateWindow(640, 480, "08 - textura", NULL, NULL);
	if (!window)
	{
		printf("Nelze inicializovat kontext\n");
		glfwTerminate();
		return -1;
	}
	/* Make the window's context current */
	glfwMakeContextCurrent(window);
	printf("%s\n", glGetString(GL_VERSION));

	/* Load modern OpenGL functions */
	glewExperimental = GL_TRUE;
	glewInit();

	/* events */
	glfwSetKeyCallback(window, key_callback);
	glfwSetMouseButtonCallback(window, mouse_button_callback);
	glfwSetCursorPosCallback(window, cursor_position_callback);

	//glfwSwapInterval(1);

	Shader program = shader("default.vs", "default.fs");
	if (!program.Program) {
		return -1;
	}

	initVariables(&program);
	initBuffers(&program);

	// setup GL state
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	mat4x4_identity(rotationMatrix);
	useColor.ambient = 0;

	ticking(&program, window);

	uninitBuffers();

	glfwDestroyWindow(window);
	glfwTerminate();
	return 0;
}

void initVariables(Shader *program)
{
	variables.uMVMatrixLoc = program->getUniformLocation(program, "uMVMatrix");
	variables.uPMatrixLoc = program->getUniformLocation(program, "uPMatrix");
	variables.uAmbientColorLoc = program->getUniformLocation(program, "uAmbientColor");

	variables.texture1 = program->getUniformLocation(program, "texture1");
}

void initBuffers(Shader *program)
{
	glGenVertexArrays(sizeof(VAO) / sizeof(VAO[0]), VAO);
	glGenBuffers(sizeof(BO) / sizeof(BO[0]), BO);
	glGenTextures(sizeof(TEX) / sizeof(TEX[0]), TEX);

	glBindTexture(GL_TEXTURE_2D, TEX[0]);
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		int width, height, n;
		unsigned char *image = sx_stbi_load("logo.png", &width, &height, &n, 0);
		printf("w = %i, h = %i, n = %i\n", width, height, n);
		if (n == 4) {
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		}
		else {
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, image);
		}
		sx_stbi_image_free(image);
	}
	glBindTexture(GL_TEXTURE_2D, 0);

	glBindVertexArray(VAO[0]);
	{
		//vertices
		glBindBuffer(GL_ARRAY_BUFFER, BO[0]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(0);
		//textures
		glBindBuffer(GL_ARRAY_BUFFER, BO[2]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(texCoord), texCoord, GL_STATIC_DRAW);
		glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, 2 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(2);
	}
	glBindVertexArray(0); // Unbind VAO
}

void uninitBuffers(void)
{
	glDeleteVertexArrays(sizeof(VAO) / sizeof(VAO[0]), VAO);
	glDeleteBuffers(sizeof(BO) / sizeof(BO[0]), BO);
}

void ticking(Shader * program, GLFWwindow* window) {
	/* Loop until the user closes the window */
	while (!glfwWindowShouldClose(window))
	{
		animate();
		draw(program, window, VAO);
		glfwSwapBuffers(window);
		glfwPollEvents();
	}
}

void animate(void) {
	x = 0.5;
	y = 0.5;
	z = -1;
}


void draw(Shader *program, GLFWwindow* window, GLuint *VAO)
{
	int width, height;
	glfwGetFramebufferSize(window, &width, &height);
	glViewport(0, 0, width, height);

	glClearColor(1.f, 1.f, 1.f, 1.f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	program->Use(program);

	mat4x4 uMVMatrix, uPMatrix;

	mat4x4_perspective(uPMatrix, degToRad(45.0), ((GLfloat)width) / height, 1.0, 100.0);
	mat4x4_translate(uMVMatrix, 0.0, 0.0, -5.0);
	mat4x4_mul(uMVMatrix, uMVMatrix, rotationMatrix);

	glUniformMatrix4fv(variables.uMVMatrixLoc, 1, GL_FALSE, (const GLfloat *)uMVMatrix);
	glUniformMatrix4fv(variables.uPMatrixLoc, 1, GL_FALSE, (const GLfloat *)uPMatrix);

	if (useColor.ambient) {
		glUniform3f(variables.uAmbientColorLoc, 0.4, 0.4, 0.4);
	}
	else {
		glUniform3f(variables.uAmbientColorLoc, 0.0, 0.0, 0.0);
	}

	glBindTexture(GL_TEXTURE_2D, TEX[0]);

	// Vykresli
	glBindVertexArray(VAO[0]);
	glDrawArrays(GL_TRIANGLES, 0, sizeof(vertices) / sizeof(vertices[0]) / 3);
	glBindVertexArray(0);
}

/*------------------------------------------------*/
